<?php

namespace Drupal\synpay\Plugin\Synpay;

use YooKassa\Client;
use YooKassa\Model\PaymentStatus;
use YooKassa\Model\NotificationEventType;
use YooKassa\Model\Notification\NotificationSucceeded;
use YooKassa\Request\Payments\Payment\CreateCaptureRequest;
use YooKassa\Model\Notification\NotificationWaitingForCapture;
use Drupal\Component\Serialization\Json;
use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides a Yandex Gateway.
 *
 * @SynpayAnnotation(
 *   id = "yookassa",
 *   title = @Translation("Ykassa"),
 * )
 */
class SynpayYandex extends SynpayPluginBase implements SynpayPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = NULL;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $this->client = new Client();
    $this->client->setAuth($this->user, $this->pass);

  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, int $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $host = \Drupal::request()->getSchemeAndHttpHost();
    $return = "$host/synpay/return_direct/yookassa";
    $params = [
      'failUrl' => "$host/synpay/return_direct/yookassa?fail",
    ];
    $order = Uuid::uuid4()->getHex()->toString();
    $order_amount = $total * 100;
    try {
      $result = $this->client->registerOrder($order, $order_amount, $return, $params);
      $redirect = $result['formUrl'];
      $response = new TrustedRedirectResponse($redirect);
      $response->send();
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::messenger()->addWarning(t('Check login/password to your payment gateway!'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    $host = \Drupal::request()->getSchemeAndHttpHost();
    // $return = "$host/synpay/return/yookassa";

    try {
      $response = $this->client->createPayment([
        'amount' => [
          'value' => $order_amount / 100,
          'currency' => 'RUB',
        ],
        'payment_method_data' => [
          'type' => 'bank_card',
        ],
        'confirmation' => [
          'type' => 'redirect',
          // 'return_url' => $return,
          'return_url' => $params['returnUrl'],
        ],
        'capture' => TRUE,
        'description' => "Заказ №$order_id",
      ],
      uniqid('', TRUE),
      );
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::logger('synpay_sberbank')->error($message);
      return FALSE;
    }
    return [
      'orderId' => $response->getId(),
      // 'orderId' => $order_id,
      'formUrl' => $response->getConfirmation()->getConfirmationUrl(),
    ];
  }

  /**
   * Pay callback.
   */
  public function callback(Request $request) {
    $rawBody = $request->getContent();
    $notificationData = Json::decode($rawBody);
    $notificationModel = ($notificationData['event'] === NotificationEventType::PAYMENT_SUCCEEDED)
            ? new NotificationSucceeded($notificationData)
            : new NotificationWaitingForCapture($notificationData);
    $paymentResponse = $notificationModel->getObject();
    $result['remote_id'] = $paymentResponse->id;
    switch ($paymentResponse->status) {
      case 'succeeded':
        $result['status'] = 'completed';
        break;

      default:
        $result['status'] = 'any';
        break;
    }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onReturn(string $remote_id) {
    $result = $this->client->getOrderStatus($remote_id);
    $rawBody = $request->getContent();
    \Drupal::logger(__FUNCTION__ . __LINE__)->notice(
      '@j', ['@j' => json_encode($rawBody ?? '')]
    );
    // dsm([[[$rawBody]]]);
    // switch ($result['orderStatus']) {
    //   case OrderStatus::DEPOSITED:
    //     $result['status'] = 'completed';
    //     break;
    //
    //   default:
    //   case OrderStatus::DECLINED:
    //     $result['status'] = 'authorization_voided';
    // }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onNotify(Request $request) {
    $result = $this->client->getOrderStatus($remote_id);
    $rawBody = $request->getContent();
    \Drupal::logger(__FUNCTION__ . __LINE__)->notice(
      '@j', ['@j' => json_encode($rawBody ?? '')]
    );
    dsm([[[$rawBody]]]);
    $this->log('Notification: ' . $rawBody);
    $notificationData  = Json::decode($rawBody);
    $notificationModel = ($notificationData['event'] === NotificationEventType::PAYMENT_SUCCEEDED)
            ? new NotificationSucceeded($notificationData)
            : new NotificationWaitingForCapture($notificationData);
    $apiClient         = $this->apiClient;
    $paymentResponse   = $notificationModel->getObject();
    $paymentId         = $paymentResponse->id;
    $payment_storage   = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments          = $payment_storage->loadByProperties(['remote_id' => $paymentId]);
    if (!$payments) {
      return new Response('Bad request', 400);
    }
    /** @var \Drupal\commerce_payment\Entity\Payment $payment */
    $payment = reset($payments);
    /** @var \Drupal\commerce_order\Entity\Order $order */
    $order = $payment->getOrder();
    if (!$order) {
      return new Response('Order not found', 404);
    }

    $paymentInfo = $apiClient->getPaymentInfo($paymentId);
    $this->log('Payment info: ' . Json::encode($paymentInfo));

    $state = $order->getState()->value;
    if ($state !== 'completed') {
      switch ($paymentInfo->status) {
        case PaymentStatus::WAITING_FOR_CAPTURE:
          $captureRequest  = CreateCaptureRequest::builder()->setAmount($paymentInfo->getAmount())->build();
          $captureResponse = $apiClient->capturePayment($captureRequest, $paymentId);
          $this->log('Payment info after capture: ' . Json::encode($captureResponse));
          if ($captureResponse->status == PaymentStatus::SUCCEEDED) {
            $payment->setRemoteState($paymentInfo->status);
            $order->state = 'completed';
            $order->setCompletedTime(\Drupal::time()->getRequestTime());
            $order->save();
            $payment->save();
            $this->log('Payment completed');

            return new Response('Payment completed', 200);
          }
          elseif ($captureResponse->status == PaymentStatus::CANCELED) {
            $payment->setRemoteState($paymentInfo->status);
            $payment->save();
            $this->log('Payment canceled');

            return new Response('Payment canceled', 200);
          }
          break;

        case PaymentStatus::PENDING:
          $payment->setRemoteState($paymentInfo->status);
          $payment->save();
          $this->log('Payment pending');

          return new Response(' Payment Required', 402);

        case PaymentStatus::SUCCEEDED:
          $payment->setRemoteState($paymentInfo->status);
          $order->state = 'completed';
          $order->setCompletedTime(\Drupal::time()->getRequestTime());
          $order->save();
          $payment->save();
          $this->log('Payment complete');

          return new Response('Payment complete', 200);

        case PaymentStatus::CANCELED:
          $payment->setRemoteState($paymentInfo->status);
          $payment->save();
          $this->log('Payment canceled');

          return new Response('Payment canceled', 200);
      }
    }
    return $result;
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayYandex')->info($message);
  }

}
