<?php

namespace Drupal\synpay\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;

/**
 * Order registration and redirection to payment URL.
 */
class OffsitePaymentForm extends PaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    // If payment is not have ID, we force save it, since it used in $order_id
    // which is required for payment.
    if ($payment->isNew()) {
      $payment->save();
    }
    $mode = $this->plugin->getMode();
    $config = $this->getPluginConfig($payment);
    $currency = $this->getPluginCurrencyCode($payment);
    // Use Payment ID instead of actual order id for good API-ID increment.
    $pid = $payment->id();
    $order_id = "{$config['order_id_prefix']}$pid{$config['order_id_suffix']}";
    $order_amount = (int) ($payment->getAmount()->getNumber() * 100);
    $params = [
      'currency' => $currency,
      'returnUrl' => $form['#return_url'],
      'failUrl' => $form['#cancel_url'],
      'orderBundle' => [
        'cartItems' => [
          'items' => [],
        ],
      ],
    ];
    $synpay = \Drupal::service('synpay.gateway')->init($mode, $config);
    $items = [];
    $c = 0;
    foreach ($payment->getOrder()->order_items as $item) {
      $order_item = $item->entity;
      $quantity = round($order_item->quantity->value);
      $price = round($order_item->unit_price->number);
      $total = round($order_item->total_price->number);
      $variation = $order_item->purchased_entity->entity;

      switch ($synpay->gateway->getPluginId()) {
        case 'robokassa':
          $items[] = [
            'name' => $order_item->title->value,
            'quantity' => $quantity,
            'sum' => $total,
            'tax' => 'none',
            // 'nomenclature_code' => $variation->id(),
          ];
          break;

        default:
          $items[] = [
            'positionId' => $c++,
            'name' => $order_item->title->value,
            'itemCode' => $variation->id(),
            'quantity' => [
              'value' => $quantity,
              'measure' => 'шт',
            ],
            'itemPrice' => $price * 100,
            'itemAmount' => $total * 100,
          ];
          break;
      }
    }

    $adjustmentItems = $this->getAdjustmentItems($payment);
    switch ($synpay->gateway->getPluginId()) {
      case 'robokassa':
        foreach ($adjustmentItems as $adjustmentItem) {
          if ($adjustmentItem['itemAmount'] != 0) {
            $items[] = [
              'name' => $adjustmentItem['name'],
              'quantity' => $adjustmentItem['quantity']['value'],
              'sum' => $adjustmentItem['itemAmount'] / 100,
              // 'payment_object' => 'service',
              'tax' => 'none',
              // 'nomenclature_code' => $adjustmentItem['itemCode'],
            ];
          }
        }
        break;

      default:
        foreach ($adjustmentItems as $adjustmentItem) {
          if ($adjustmentItem['itemAmount'] != 0) {
            $adjustmentItem['positionId'] = $c++;
            $items[] = $adjustmentItem;
          }
        }
        break;
    }

    $params['orderBundle']['cartItems']['items'] = $items;
    if ($synpay->gateway->getPluginId() == 'sbercredit') {
      $params['orderBundle']['installments'] = [
        'productType' => 'CREDIT',
        'productID' => 10,
      ];
    }
    $pay = $synpay->registerOrder($order_id, $order_amount, $params);
    if ($pay) {
      $payment->setAuthorizedTime(time());
      $payment->setRemoteId($pay['orderId']);
      $payment->setState('authorization');
      $payment->save();
      switch ($synpay->gateway->getPluginId()) {
        case 'robokassa':
          return $this->buildRedirectForm($form, $form_state, $pay['url'], $pay['data'], self::REDIRECT_POST);

        default:
          return $this->buildRedirectForm($form, $form_state, $pay['formUrl'], [], self::REDIRECT_GET);
      }
    }
    else {
      $order = $payment->getOrderId();
      \Drupal::logger('synpay')->error("Order: $order / Payment $pid fails.");
      // Mark payment is failed.
      $payment->setState('authorization_voided');
      $payment->save();
    }
    return $form;
  }

  /**
   * Получить корректировки цены.
   */
  private function getAdjustmentItems(PaymentInterface $payment) : array {
    $order = $payment->getOrder();
    $adjustments = array_map(function ($adjustment) {
      $amount = $adjustment->getAmount()
        ->getNumber();
      if (empty($amount)) {
        return NULL;
      }
      return [
        'name' => $adjustment->getLabel(),
        'itemCode' => implode('-', [
          $adjustment->getType(),
          $adjustment->getSourceId(),
        ]),
        'quantity' => [
          'value' => 1,
          'measure' => 'шт',
        ],
        'itemPrice' => $amount * 100,
        'itemAmount' => $amount * 100,
      ];
    }, $order->getAdjustments());
    return array_filter($adjustments);
  }

  /**
   * Plugin currency.
   */
  private function getPluginCurrencyCode(PaymentInterface $payment) : string {
    $currency_code = $payment->getAmount()->getCurrencyCode();
    $storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    /** @var \Drupal\commerce_price\Entity\CurrencyInterface $currency */
    $currency = $storage->load($currency_code);
    return $currency->getNumericCode();
  }

  /**
   * Plugin config.
   */
  private function getPluginConfig(PaymentInterface $payment) : array {
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configs = $payment_gateway_plugin->getConfiguration();
    // Get username and password for payment method.
    // $gw = $configs['gw'];
    return [
      'gateway' => $configs['gateway'],
      'order_id_prefix' => "",
      'order_id_suffix' => "",
    ];
  }

}
